import bs4
import requests


class PageObject(object):
    _url: str
    _soup: bs4.BeautifulSoup
    _response: requests.Response
    _headers: dict | None

    def __init__(self, **kwargs):
        self.url = kwargs.get("url")
        self.soup = None
        self.response = None
        self.headers = None

    @property
    def url(self):
        return self._url

    @url.setter
    def url(self, url: str):
        self._url = url

    @property
    def soup(self):
        return self._soup

    @property
    def response(self):
        return self._response

    @response.setter
    def response(self, response: requests.Response):
        self._response = response

    @property
    def headers(self):
        return self._headers

    @headers.setter
    def headers(self, headers: dict | None):
        self._headers = headers

    @soup.setter
    def soup(self, value):
        self._soup = value

    @property
    def status_code(self):
        if self.response is None:
            return None

        return self.response.status_code

    def load(self, url: str = None):
        if url is not None:
            self.url = url

        # print(f"Loading page: {self.url}")

        if self.headers is None:
            self.response = requests.get(self.url)
        else:
            self.response = requests.get(self.url, headers=self.headers)

        self.response.raise_for_status()
        self.soup = bs4.BeautifulSoup(self.response.content, "html.parser")

        return self

    def erase(self):
        self.url = None
        self.soup = None
        self.response = None

        return self

    def __repr__(self):
        return f"<PageObject(url='{self.url}')>"



def slugify(value: str | None) -> str | None:
    """
    Converts the given value into a slug.

    :param value:
    :return: A slug
    """
    if value is None:
        return None

    value = value.strip()
    value = value.lower()
    value = value.replace(" ", "-")
    value = value.replace(".", "")
    value = value.replace("'", "")

    return value


def urljoin(base: str | None, path: str | None) -> str:
    """
    Joins the given base URL and path.
    Args:
        base:
        path:

    Returns: Joined URL
    """
    if base is None:
        raise ValueError("Undefined base URL!")

    if path is None:
        raise ValueError("Undefined path!")

    if len(base) == 0:
        if len(path) == 0:
            return ""
        else:
            return path

    if base.endswith("/"):
        base = base[:-1]

    if path.startswith("/"):
        path = path[1:]

    return str(f"{base}/{path}")

def get_href_from_anchor(anchor):
    if anchor is None:
        return None

    if anchor.name != "a":
        anchor = anchor.find("a")

    if anchor is None:
        return None

    href = anchor.get("href")
    if href is None:
        return None

    href = href.strip()
    if len(href) == 0:
        return None

    return href


def get_text_from_anchor(anchor):
    if anchor is None:
        return None

    if anchor.name != "a":
        anchor = anchor.find("a")

    if anchor is None:
        return None

    text = anchor.text.strip()
    if len(text) == 0:
        return None

    return text



from bs4 import element


def get_href(tag: element) -> str | None:
    if tag is None:
        return None

    if tag.name != "a":
        tag = tag.find("a")

    if tag is None:
        return None

    href = tag.get("href")
    if href is None:
        return None

    href = href.strip()
    if len(href) == 0:
        return None

    return href


def get_text(tag: element) -> str | None:
    if tag is None:
        return None

    if tag.name != "a":
        tag = tag.find("a")

    if tag is None:
        return None

    text = tag.text.strip()
    if len(text) == 0:
        return None

    return text



from enum import Enum


class Gender(Enum):
    # __order__ = "ALL Male Female"
    All = 0
    Male = 1
    Female = 2

    def __repr__(self):
        return self

    def __str__(self):
        return self.name.lower()


def string_to_gender(value: str | None) -> Gender:
    if value is None:
        raise ValueError("Undefined gender string!")

    value = value.strip()

    if len(value) == 0:
        raise ValueError("Empty gender string!")

    value = value.lower()

    if value in ["all"]:
        return Gender.All
    elif value in ["m", "male"]:
        return Gender.Male
    elif value in ["f", "female"]:
        return Gender.Female
    else:
        raise ValueError(f"Invalid gender string '{value}'!")
